import EventHandler from './EventHandler';

/**
 * Represents an object that manages the Annotations that appear on a Document's pages when displayed in a DocumentViewer.
 * @memberof Core
 * @extends Core.EventHandler
 */
class AnnotationManager extends EventHandler {
  /**
   * Creates a new instance of AnnotationManager.
   * @param {object} options - A map of properties to set.
   * @property {Core.DocumentViewer} container - The target element of annotation control points.
   * @property {array} annotations - The list of the annotations of the document.
   * @property {object} viewport - The viewport of the document.
   */
  constructor(options) {
    this.container = options.container || null;
    this.annotations = options.annotations || [];
    this.viewport = options.viewport;
  }

  /**
   * The different action types for the annotationChanged event.
   * @property {string} ADD - When the annotationChanged event triggered due to adding annotations
   * @property {string} DELETE - When the annotationChanged event triggered due to deleting annotations
   * @property {string} MODIFY - When the annotationChanged event triggered due to modifying annotations
   */
  static get AnnotationChangedActions() {
    return {
      ADD: 'ADD',
      DELETE: 'DELETE',
      MODIFY: 'MODIFY'
    };
  }

  /**
   * @property {string} ANNOTATION_SELECTED - Core.AnnotationManager.annotationSelected
   * @property {string} ANNOTATION_DESELECTED - Core.AnnotationManager.annotationDeselected
   * @property {string} ANNOTATION_DOUBLE_CLICKED - Core.AnnotationManager.annotationDoubleClicked
   * @property {string} ANNOTATION_CHANGED - Core.AnnotationManager.annotationChanged
   * @property {string} ANNOTATION_NUMBERING_UPDATED - Core.AnnotationManager.annotationNumberingUpdated
   * @property {string} UPDATE_ANNOTATION_PERMISSION - Core.AnnotationManager.updateAnnotationPermission
   * @property {string} ANNOTATIONS_DRAWN - Core.AnnotationManager.annotationsDrawn
   * @property {string} ANNOTATION_HIDDEN - Core.AnnotationManager.annotationHidden
   * @property {string} SET_NOTE_TEXT - Core.AnnotationManager.setNoteText
   * @property {string} ADD_REPLY - Core.AnnotationManager.addReply
   * @property {string} DELETE_REPLY - Core.AnnotationManager.deleteReply
   * @property {string} FIELD_CHANGED - Core.AnnotationManager.fieldChanged
   * @property {string} FILE_ATTACHMENT_DATA_AVAILABLE - Core.AnnotationManager.fileAttachmentDataAvailable
   */
  static get Events() {
    return {
      ANNOTATION_SELECTED: 'ANNOTATION_SELECTED',
      ANNOTATION_DESELECTED: 'ANNOTATION_DESELECTED',
      ANNOTATION_DOUBLE_CLICKED: 'ANNOTATION_DOUBLE_CLICKED',
      ANNOTATION_CHANGED: 'ANNOTATION_CHANGED',
      ANNOTATION_NUMBERING_UPDATED: 'ANNOTATION_NUMBERING_UPDATED',
      UPDATE_ANNOTATION_PERMISSION: 'UPDATE_ANNOTATION_PERMISSION',
      ANNOTATIONS_DRAWN: 'ANNOTATIONS_DRAWN',
      ANNOTATION_HIDDEN: 'ANNOTATION_HIDDEN',
      SET_NOTE_TEXT: 'SET_NOTE_TEXT',
      ADD_REPLY: 'ADD_REPLY',
      DELETE_REPLY: 'DELETE_REPLY',
      FIELD_CHANGED: 'FIELD_CHANGED',
      FILE_ATTACHMENT_DATA_AVAILABLE: 'FILE_ATTACHMENT_DATA_AVAILABLE'
    };
  }

  /**
   * Represents the types of rotations available to perform on an annotation.
   * @property {string} SNAP_ROTATION - Rotate an annotation in discrete steps, such as 45 or 90 degrees
   * @property {string} FREEFORM_ROTATION - Rotate an annotation continuously
   */
  static get RotationTypes() {
    return {
      SNAP_ROTATION: 'SNAP_ROTATION',
      FREEFORM_ROTATION: 'FREEFORM_ROTATION'
    };
  }

  /**
   * Represents the types of tabbing orders available to set on widgets in the document.
   * @property {string} STRUCTURE - Order widgets according to the order found in the internal structure. This is default
   * @property {string} ROW - Order widgets in terms of rows
   * @property {string} COLUMN - Order widgets in terms of columns
   */
  static get TabbingOrders() {
    return {
      STRUCTURE: 'STRUCTURE',
      ROW: 'ROW',
      COLUMN: 'COLUMN'
    };
  }

  /**
   * Adds the specified annotation to the managed list of annotations.
   * @param {Core.Annotations.Annotation|Array.<Core.Annotations.Annotation>} annotation - An array of annotations.
   * @param {object} [options] - An object that can contain the following optional parameters.
   * @param {boolean} [options.imported=false] - Whether the annotations were imported from another source or not
   * @param {boolean} [options.isUndoRedo=false] - Whether the annotation change was caused by undo/redo or not
   * @param {boolean} [options.autoFocus=false] - Whether the annotation's text input should be automatically focused if applicable. Relevant to FreeText and Callout annotations.
   * @param {string} [options.source] - What type of action cause this event (i.e. "redactionApplied")
   */
  addAnnotation(annotation, options) {}

  /**
   * Adds the specified annotations to the managed list of annotations.
   * @param {Array.<Core.Annotations.Annotation>} annotations - An array of annotations.
   * @param {object} [options] - An object that can contain the following optional parameters.
   * @param {boolean} [options.imported=false] - Whether the annotations were imported from another source or not
   * @param {boolean} [options.isUndoRedo=false] - Whether the annotation change was caused by undo/redo or not
   * @param {boolean} [options.autoFocus=false] - Whether the annotation's text input should be automatically focused if applicable. Relevant to FreeText and Callout annotations.
   * @param {string} [options.source] - What type of action cause this event (i.e. "redactionApplied")
   */
  addAnnotations(annotations, options) {}

  /**
   * Creates an annotation that replies to the passed in annotation. Annotation replies are sticky note annotations.
   * @param {Core.Annotations.Annotation} annotation - The annotation to add a reply to
   * @param {string} initialText - The initialText for the annotation, defaults to the empty string
   * @returns {Core.Annotations.StickyAnnotation} The created annotation reply
   */
  createAnnotationReply(annotation, initialText) {}

  /**
   * Deletes the specified annotation in the managed list of annotations. If an annotation is successfully deleted, the annotationChanged event will be fired with a "delete" action.
   * @param {Core.Annotations.Annotation|Array.<Core.Annotations.Annotation>} annotation - An instance of Annotation.
   * @param {object} [options] - An object that can contain the following optional parameters.
   * @param {boolean} [options.imported=false] - Whether the annotations were imported from another source or not
   * @param {boolean} [options.isUndoRedo=false] - Whether the annotation change was caused by undo/redo or not
   * @param {boolean} [options.autoFocus=false] - Whether the annotation's text input should be automatically focused if applicable. Relevant to FreeText and Callout annotations.
   * @param {string} [options.source] - What type of action cause this event (i.e. "redactionApplied")
   */
  deleteAnnotation(annotation, options) {}

  /**
   * Deletes the specified annotations in the managed list of annotations. If an annotation is successfully deleted, the annotationChanged event will be fired with a "delete" action.
   * @param {Array.<Core.Annotations.Annotation>} annotations - An array of annotations.
   * @param {object} [options] - An object that can contain the following optional parameters.
   * @param {boolean} [options.imported=false] - Whether the annotations were imported from another source or not
   * @param {boolean} [options.isUndoRedo=false] - Whether the annotation change was caused by undo/redo or not
   * @param {boolean} [options.autoFocus=false] - Whether the annotation's text input should be automatically focused if applicable. Relevant to FreeText and Callout annotations.
   * @param {string} [options.source] - What type of action cause this event (i.e. "redactionApplied")
   */
  deleteAnnotations(annotations, options) {}

  /**
   * Exports all annotations as an XFDF (XML) string
   * @param {object} [options] - Options for the export. Set options.widgets or options.links or options.fields to false to disable exporting of them.
   * @param {Array.<Core.Annotations.Annotation>} [options.annotList] - An array of annotations to only export the XFDF for those particular annotations.
   * @param {Array.<Core.Annotations.Annotation>} [options.annotationList] - An array of annotations to only export the XFDF for those particular annotations.
   * @param {boolean} [options.widgets] - Whether to export widget information
   * @param {boolean} [options.links] - Whether to export links information
   * @param {boolean} [options.fields] - Whether to export fields information
   * @param {boolean} [options.useDisplayAuthor] - Whether to export annotations with the Display Author name from annotationManager.getDisplayAuthor()
   * @param {boolean} [options.generateInlineAppearances] - Whether to generate custom appearance strings for annotations that have an appearance generated on export (e.g. freetext). Default: true
   * @returns {Promise.<string>} Returns a promise that resolves with the XFDF (XML) annotations as a string
   */
  exportAnnotations(options) {}

  /**
   * Gets an annotation object by the annotation's ID.
   * @param {*} id - The ID of the annotation.
   * @param {*} annotationList - Optionally pass your own array of annotations to search in
   */
  getAnnotationById(id, annotationList) {}

  /**
   * Gets the list of all annotations managed by the AnnotationManager.
   * @returns {Array.<Core.Annotations.Annotation>} An array of Annotations.
   */
  getAnnotationsList() {}

  /**
   * Returns the user name of the current user.
   * @returns {string} The user name of the current user.
   */
  getCurrentUser() {}

  /**
   * Annotations may set the author to a unique id which isn't suitable for display in the UI. this function gets the author name of the annotation that should be displayed.
   * @param {string} annotationUserId - Annotation userId
   * @returns {string} The display author name
   */
  getDisplayAuthor(annotationUserId) {}

  /**
   * Gets a map of registered annotations. This can be modified directly, instead of using AnnotationManager#registerAnnotationType and AnnotationManager#deregisterAnnotationType.
   * @returns {Object} a JavaScript object containing a key-value map, where the key is the annotation element name and the value is an array of Annotation classes.
   */
  getRegisteredAnnotationTypes() {}

  /**
   * Hides the specified annotation.
   * @param {Core.Annotations.Annotation} annot - The annotation to hide 
   */
  hideAnnotation(annot) {}

  /**
   * Hides all of the annotations in the list
   * @param {Array.<Core.Annotations.Annotation>} annots - Array of annotations to hide
   */
  hideAnnotations(annots) {}

  /**
   * Loads XFDF annotations into the viewer
   * @param {string} xfdfString - The XFDF annotations as a string
   * @param {object} [options] - The options for importing
   * @param {number} [options.batchSize] - The number of annotations to import in each batch (default 100)
   * @param {number} [options.batchDelay] - The amount of time in milliseconds to delay between importing each batch (default 0)
   * @param {Core.Annotations.Annotation|Array.<Core.Annotations.Annotation>} [options.batchSize] - The type of existing annotations that will be removed before import starts (default [])
   * @returns {Promise.<any>} Returns a promise that resolves with the annotations have been imported
   */
  importAnnotations(xfdfString, options) {}

  /**
   * Redraws the specified annotation. Note that the entire annotation canvas for the page will be redrawn.
   * @param {Core.Annotations.Annotation} annotation - The annotation to be redrawn.
   */
  redrawAnnotation(annotation) {}

  /**
   * Set the styles for the annotation
   * @param {Core.Annotations.Annotation} annotation - an annotation of which the styles will be changed.
   * @param {object|function} newStyles - if an object is used, it should contain properties of the new styles. If a function is used, the current styles will be passed as its argument and the function should return an object which contains properties of the new styles. Example of valid properties: StrokeColor, TextColor, FillColor, FontSize, Opacity, StrokeThickness, Precision, Scale, OverlayText, Style and Dashes.
   */
  setAnnotationStyles(annotation, newStyles) {}

  /**
   * Shows the annotation
   * @param {Core.Annotations.Annotation} annot - The annotation to show
   */
  showAnnotation(annot) {}

  /**
   * Shows all of the annotations in the list
   * @param {Array.<Core.Annotations.Annotation>} annots - Array of annotations to show
   */
  showAnnotations(annots) {}

  /**
   * Redraws the annotations on the same page as the specified annotation if the annotation has been added.
   * @param {Core.Annotations.Annotation} annotation - An instance of Annotation.
   */
  updateAnnotation(annotation) {}

  /**
   * Updates the annotation state. This is done by creating an annotation that replies to the passed in annotation.
   * @param {Core.Annotations.Annotation} annotation - The annotation to add a reply to
   * @param {string} state - Annotations may have an author-specific state associated with them. Valid states are for Review stateModel: Accepted, Rejected, Cancelled, Completed, None. Default is None. Valid states are for Marked stateModel: Marked and Unmarked. Default is Unmarked.
   * @param {string} stateModel - Gets or sets the stateModel of the annotation. stateModel has two statemodels: Marked, Review. Default is Review.
   * @param {string} message - Message to be set when the state is updated.
   * @returns {Core.Annotations.StickyAnnotation} The created annotation reply
   */
  updateAnnotationState(annotation, state, stateModel, message) {}

  /**
   * Triggered when a reply has been added to an annotation
   * @event Core.AnnotationManager#addReply
   * @param {Core.Annotations.Annotation} annotation - The annotation that was added
   * @param {Core.Annotations.Annotation} parent - The annotation that is the direct parent of the first annotation
   * @param {Core.Annotations.Annotation} root - The annotation that is the root parent of the first annotation (may be the same as parent)
   */
  addReply() {}

  /**
   * Triggered when an annotation or annotations have been changed (added, deleted, modified). Attach like annotManager.addEventListener('annotationChanged', callback)
   * @event Core.AnnotationManager#annotationChanged
   * @param {Array.<Core.Annotations.Annotation>} annotations - The annotations that were changed
   * @param {string} action - The action that occurred (add, delete, modify)
   * @param {Core.AnnotationManager.AnnotationChangedInfoObject} info - An object containing extra information about the annotationChanged event
   */
  annotationChanged() {}

  /**
   * Triggered after annotations have been deselected.
   * @event Core.AnnotationManager#annotationDeselected
   * @param {Array.<Core.Annotations.Annotation>} annotationList - List of annotations that have been deselected.
   */
  annotationDeselected() {}

  /**
   * Triggered after an annotation has been double clicked
   * @event Core.AnnotationManager#annotationDoubleClicked
   * @param {Core.Annotations.Annotation} annotation - The annotation that has been double clicked
   */
  annotationDoubleClicked() {}

  /**
   * Triggered after annotations have been hidden/shown.
   * @event Core.AnnotationManager#annotationHidden
   * @param {Array.<Core.Annotations.Annotation>} annotationList - List of annotations that were hidden or shown
   * @param {boolean} hidden - Whether the annotations have been hidden or shown
   */
  annotationHidden() {}

  /**
   * Triggered after annotations have been drawn for a page
   * @event Core.AnnotationManager#annotationsDrawn
   * @param {object} pageNumber - The page number of all the annotations that were just drawn
   */
  annotationsDrawn() {}

  /**
   * Triggered when a reply has been deleted from an annotation
   * @event Core.AnnotationManager#deleteReply
   * @param {Core.Annotations.Annotation} annotation - The annotation that was deleted
   * @param {Core.Annotations.Annotation} root - The annotation that is the root parent of the first annotation
   */
  deleteReply() {}
}

/**
 * @typedef {object} Core.AnnotationManager.AnnotationChangedInfoObject
 * @property {boolean} imported - A boolean that will be true if the annotation change is the result of importing annotations using importAnnotations, importAnnotationCommand or if the imported parameter is set to true when calling addAnnotations or deleteAnnotations
 * @property {boolean} isUndoRedo - A boolean that will be true if the annotation change is the result of an undo or redo action
 * @property {string} source - An optional string that indicates what type of action caused this event (i.e. "redactionApplied") Core.AnnotationManager.AnnotationChangedSources
 */

/**
 * Sources for what triggered the annotationChanged event
 * @typedef {string} Core.AnnotationManager.AnnotationChangedSources
 * @property {string} MOVE - Source for when the annotation was moved
 * @property {string} RESIZE - Source for when the annotation was resized
 * @property {string} ROTATE - Source for when the annotation was rotated
 * @property {string} TEXT_CHANGED - Source for when the annotation's text was changed (FreeText)
 * @property {string} NOTE_CHANGED - Source for when the annotation's note was changed
 * @property {string} DRAGGING_ACROSS_PAGES - Source for when the annotation was dragged across pages
 */