/**
 * Represents an annotation.
 * @memberof Core.Annotations
 */
class Annotation {

  /**
   * An object containing the different display formats for measurement annotations.
   * @property {string} DECIMAL - Decimal format eg. 12.75
   * @property {string} FRACTION - Fraction format eg. 12 3/4
   * @property {string} ROUND - Decimal format eg. 12.8
   * @property {string} TRUNCATE - Decimal format eg. 12.7
   */
  static get MeasurementDisplayFormats() {
    return {
      DECIMAL: 'DECIMAL',
      FRACTION: 'FRACTION',
      ROUND: 'ROUND',
      TRUNCATE: 'TRUNCATE'
    };
  }

  /**
   * An object containing the different measurement systems for measurement annotations.
   * @property {string} METRIC - The Metric measurement system
   * @property {string} IMPERIAL - The Imperial measurement system
   * @property {string} TYPOGRAPHIC - The Typographic measurement system
   */
  static get MeasurementSystems() {
    return {
      METRIC: 'The Metric measurement system',
      IMPERIAL: 'The Imperial measurement system',
      TYPOGRAPHIC: 'The Typographic measurement system'
    };
  }

  /**
   * The base class for all annotations
   * @param {object} [initializer] - A map of values to auto-initialize certain properties of the annotation. You can only initialize properties defined on the annotation under the Members section (unless specified otherwise).
   */
  constructor(initializer) {
    /** The author of the annotation. */
    this.author = initializer.author;

    /**
     * Gets or sets the annotation blend mode.
     * 
     * **THIS PROPERTY IS EXPERIMENTAL, NOT CURRENTLY PERSISTED AND ONLY WORKS WITHIN WEBVIEWER**
     */
    this.BlendMode = initializer.BlendMode;

    /** Gets or sets the annotation's stroke color. */
    this.color = initializer.color;

    /** Gets or sets the date that the annotation was last modified. */
    this.DateModified = initializer.DateModified;

    /** Gets or sets the name of the annotation element in XFDF.
     * 
     * This property cannot be initialized through the initializer.
     */
    this.elementName = initializer.elementName;

    /** Gets or sets the height of the annotation. */
    this.Height = initializer.Height;

    /** Gets or sets whether the annotation is hidden. */
    this.Hidden = initializer.Hidden;

    /** A unique identifier for the annotation. Corresponds to the name attribute in XFDF. */
    this.Id = initializer.Id;

    /** The Id of the annotation that this annotation is replying to or the parent annotation it is grouped with. */
    this.InReplyTo = initializer.InReplyTo;

    /** Gets or sets whether the annotation is invisible, only if it is an unknown annotation type. Generally for hiding annotations you should use "Hidden". */
    this.Invisible = initializer.Invisible;

    /** Gets or sets whether the annotation has been added since the last time the AnnotCommand was retrieved. */
    this.IsAdded = initializer.IsAdded;

    /** Gets or sets whether any parts of the annotation drawn outside of the rect are clickable. */
    this.IsClickableOutsideRect = initializer.IsClickableOutsideRect;

    /** Gets or sets whether the annotation has been deleted since the last time the AnnotCommand was retrieved. */
    this.IsDeleted = initializer.IsDeleted;

    /** Gets or sets whether the annotation has been modified since the last time the AnnotCommand was retrieved. */
    this.IsModified = initializer.IsModified;

    /** Gets or sets the page number of a document that the annotation appears on. */
    this.PageNumber = initializer.PageNumber;

    /** Gets or sets whether the annotation should be displayed when printing the page. */
    this.Printable = initializer.Printable;

    /** Gets or sets whether the annotation is readonly or not. If it's readonly it can't be edited or deleted. */
    this.ReadOnly = initializer.ReadOnly;

    /** The type of reply. */
    this.ReplyType = initializer.ReplyType;

    /** Gets or sets the annotation's clockwise rotation in degrees. Valid values are 0, 90, 180 and 270. Only applies to Stamp, FreeText and Caret annotations. */
    this.Rotation = initializer.Rotation;

    /** Enables or disables the annotation's rotation control. */
    this.RotationControlEnabled = initializer.RotationControlEnabled;

    /** Gets or sets a selection model constructor for how an annotation gets selected. */
    this.selectionModel = initializer.selectionModel;

    /** Represents the subject of the annotation. Default is the annotation's type. */
    this.Subject = initializer.Subject;

    /** Gets or sets whether the ToggleNoView flag is set on the annotation. */
    this.ToggleNoView = initializer.ToggleNoView;

    /** The name of the tool that is used to create this annotation. */
    this.ToolName = initializer.ToolName;

    /** Gets or sets the width of the annotation. */
    this.Width = initializer.Width;

    /** Gets or sets the annotation's x-axis position. */
    this.X = initializer.X;

    /** Gets or sets the annotation's y-axis position. */
    this.Y = initializer.Y;
  }

  /**
   * Draws the annotation on the provide canvas context, relative to the page. The point (0,0) coresponds to the top left corner of the page.
   * @param {CanvasRenderingContext2D} ctx - The canvas context prepared to be drawn on.
   * @param {object} pageMatrix - The page matrix for the page that the annotation is on. You can get this object by calling getPageMatrix on the document object.
   */
  draw(ctx, pageMatrix) {}

  /**
   * Gets the text content for the annotation. Contents may be displayed in an annotation's popup or directly on the page (in the case of FreeTextAnnotation).
   * @returns {string} the text content for the annotation.
   */
  getContents() {}

  /**
   * Returns custom data associated with the given key.
   * @param {string} key - The key for which to retrieve the associated data.
   * @returns {string} value The custom data. If no data is available an empty string is returned.
   */
  getCustomData(key) {}

  /**
   * Gets the height of the annotation.
   * @returns {number} the height of the annotation.
   */
  getHeight() {}

  /**
   * Gets the page number of the annotation. Note: page number starts from 1.
   * @returns {number} The annotation's page number.
   */
  getPageNumber() {}

  /**
   * Get annotation bounding rectangle
   * @returns {Core.Math.Rect} The annotation's bounding rectangle.
   */
  getRect() {}

  /**
   * Gets the padding that will be applied by default on the annotation's rectangle.
   * @returns {number} The amount of padding
   */
  getRectPadding() {}

  /**
   * Gets the list of replies to this annotation.
   * @returns {Array.<Core.Annotations.Annotation>} The list of replies
   */
  getReplies() {}

  /**
   * Gets the status of the annotation, and returns an empty string if no status set.
   * @returns {string} The last status update.
   */
  getStatus() {}

  /**
   * Gets the width of the annotation.
   * @returns {number} The width of the annotation.
   */
  getWidth() {}

  /**
   * Gets the x position measured in page coordinates of an annotation.
   * @returns {number} the x position
   */
  getX() {}

  /**
   * Gets the y position measured in page coordinates of an annotation.
   * @returns {number} the y position
   */
  getY() {}

  /**
   * Checks if a content edit annotation is being edited and has focus.
   * @param {Core.ContentEditManager} contentEditManager - An instance of contentEditManager
   * @returns {boolean} Returns whether an annotation is on content edit focus mode or not
   */
  isInContentEditFocusMode(contentEditManager) {}

  /**
   * Gets whether the annotation is a reply to another annotation.
   * @returns {boolean} Returns true if it does reply to another annotation, false otherwise.
   */
  isReply() {}

  /**
   * Whether the annotation is visible on the document or not **when viewing**. If the Hidden or NoView flags are set, or if the annotation is a reply to another annotation then it won't be visible.
   * @returns {boolean} Whether the annotation is visible on the document or not when viewing
   */
  isVisible() {}

  /**
   * Move the annotation based on a new given Core.Math.Rect. Use this method instead of setRect when internal properties (other than x, y, width and height) need to be modified on move.
   * @param {Core.Math.Rect} rect - the new bounding rectangle
   */
  move() {}

  /**
   * Set the text content for the annotation. Note that this will not refresh the text in the UI.
   * @param {string} value - the text content to be set
   */
  setContents(value) {}

  /**
   * Sets the custom data associated with the specified key.
   * @param {string} key - The key under which to store this custom data. Data will automatically be saved in the exported XFDF and merged into the PDF when downloading.
   * @param {string} value - The custom data to store
   */
  setCustomData(key, value) {}

  /**
   * Sets the height of the annotation.
   * @param {number} value - the height of the annotation.
   */
  setHeight(value) {}

  /**
   * Sets the page number of the annotation. Note: page numbers start from 1.
   * @param {number} value - the page number to be set
   */
  setPageNumber(value) {}

  /**
   * Sets the size and location of the annotation's bounding rectangle. Use this method instead of resize when only the x, y, width and height needs to be modified. Beware: this method ignores coordinates if they are the wrong way around. (It hasn't been fixed for reasons of maintaining backward compatibility. use setRectWithNormalization instead if you want it to reverse the wrong-way-round coordinates instead of ignoring them.)
   * @param {Core.Math.Rect} rect - the new bounding rectangle
   */
  setRect(rect) {}

  /**
   * Sets the width of the annotation.
   * @param {number} value - the width of the annotation.
   */
  setWidth(value) {}

  /**
   * Sets the x position measured in page coordinates of an annotation.
   * @param {number} value - the x position
   */
  setX(value) {}

  /**
   * Sets the y position measured in page coordinates of an annotation.
   * @param {number} value - the y position
   */
  setY(value) {}
}

/**
 * an object that contains content editing properties which apply to selections of text while inline editing.
 * @typedef {Object} Core.Annotations.Annotation.ContentEditingFormat
 * @property {boolean} bold - Whether or not the selection of text is bold
 * @property {Core.Annotations.Color} color - The color of the selection of text
 * @property {boolean} italic - Whether or not the selection of text is italic
 * @property {boolean} underline - Whether or not the selection of text is underlined
 * @example
 * {
 *  bold: false,
 *  color: new instance.Core.Annotations.Color('#424242'),
 *  italic: false,
 *  underline: false,
 * }
 */

/**
 * an object that contains content style properties from a content editing annotation.
 * @typedef {Object} Core.Annotations.Annotation.ContentStyleProperties
 * @property {string} fontFamily - the font family of the text content
 * @property {string} fontSize - the font size of the text content
 * @property {string} textAlign - the alignment of the text content
 * @example
 * {
 *  fontFamily: 'SourceSansProSemiBold',
 *  fontSize: '30pt',
 *  textAlign: 'left'
 * }
 */