import AnnotationManager from './AnnotationManager';
import DocumentViewer from './DocumentViewer';

/** 
 * ComPDFKit WebViewer Instance Core namespace. The namespace containing WebViewer's core user controls, including DocumentViewer control and the Document class.
 * @namespace
 */
const Core = {
  /**
   * AnnotationManager instance
   * @see Core.AnnotationManager
   */
  annotationManager: AnnotationManager,
  
  /**
   * The types of backend workers.
   * @property {string} ASM - 'asm' Use of ASM.js worker.
   * @property {string} WASM - 'ems' Use of the WebAssembly worker (or ASM.js on non-wasm browsers).
   * @property {string} THREADED_WASM - 'wasm-threads' Use of threaded WebAssembly worker.
   */
  BackendTypes: null,
  
  /**
   * Display modes that control how the document's page are displayed
   * @property {string} Single - Displays one page at a time.
   * @property {string} Continuous - Displays all the pages in a scrolling view in one column.
   * @property {string} Facing - Displays up to two pages at a time, side by side.
   * @property {string} FacingContinuous - Displays all pages in a scrolling view in two columns.
   * @property {string} Cover - Displays all pages in a scrolling view in two columns. The first row has a single page in the second column.
   * @property {string} CoverFacing - Same as the Cover display mode except that it is not a scrolling view.
   * @property {string} Custom - Any custom display mode.
   */
  DisplayModes: null,

  /**
   * DocumentViewer instance
   * @see Core.DocumentViewer
   * @example
   * WebViewer(...)
  .then(function(instance) {
    const documentViewer = instance.Core.documentViewer;
  });
   */
  documentViewer: DocumentViewer,

  /** 
   * Represents the different Marked state values that you can apply to annotation status.
   * @property {string} Marked - Marked State
   * @property {string} Unmarked - Unmarked State
   */
  MarkedStates: null,

  /** 
   * Represents the different review state values that you can apply to annotation status.
   * @property {string} Accepted - Accepted State
   * @property {string} Rejected - Rejected State
   * @property {string} Cancelled - Cancelled State
   * @property {string} Completed - Completed State
   * @property {string} None - None State
   */
  ReviewStates: null,

  /** 
   * Represents the different state models that you can apply to annotation.
   * @property {string} Marked - Marked State
   * @property {string} Review - Review State
   */
  StateModel: null,

  /**
   * Creates an initialized Core.Document instance.
   * @param {string|File|ArrayBuffer|Blob|Core.PDFNet.PDFDoc} src - Source parameter, path/url to document or File.
   * @param {Core.CreateDocumentOptions} [options] - Create document options.
   * @returns {Promise.<Core.Document>} Promise that resolves when doc.loadAsync has been successfully called and returns Core.Document instance.
   */
  createDocument(src, options) {},

  /**
   * Set default backend type. This method can be used for forced backend type for specific workers, for example, "ems" for Emscripten worker type
   * @param {string} backendType - Value for default backend type, e.g. asm, ems, wasm-threads, jsworker.
   */
  forceBackendType(backendType) {},

  /**
   * Gets the current backend type being used.
   * @returns {string} Returns "asm" if ASM.js is being forced, "ems" if Emscripten is being used, "wasm-threads" if threaded Wasm is being used and undefined if nothing is being used yet
   */
  getCurrentPDFBackendType() {},

  /**
   * Gets an array of currently instantiated documentViewer objects.
   * @returns {Array.<Core.DocumentViewer>} Array of documentViewer objects that are being used for the UI
   * @example
   * WebViewer(...)
    .then(function(instance) {
      const documentViewerList = instance.Core.getDocumentViewers();
      documentViewerList.forEach(documentViewer => documentViewer.UI.loadDocument(doc, options));
    });
   */
  getDocumentViewers() {},

  /**
   * Gets the value of the specified hash parameter from the window URL
   * @param {string} property - The property key from hash parameter
   * @param {T} defaultValue - Default return value
   * @returns {T} A value of the hash parameter
   */
  getHashParameter(property, defaultValue) {},

  /**
   * Returns the WebViewer version.
   * @returns {string} WebViewer version
   */
  getVersion() {},

  /**
   * Begins setup of PDF Worker Object.
   * @param {string} pdfBackendType - object representing an PDF backend type ("asm", "ems" or "wasm-threads")
   * @param {*} workerHandlers - object holding event and error handlers for the worker (workerLoadingProgress, emsWorkerError).
   * @param {string} license - The license key to use for this worker. If undefined PDFNet will be initialized in demo mode.'
   * @returns a promise that will be resolved when worker transport initialization is complete.
   */
  initPDFWorkerTransports(pdfBackendType, workerHandlers, l) {}
}

/**
 * Create document options.
 * @typedef Core.CreateDocumentOptions
 * @property {string} [l] - Same as licenseKey.
 * @property {string} [licenseKey] - The license key for viewing PDF or Office files (PDF/Office only). You only need to use one of 'l' or 'licenseKey'.
 * @property {string} [docId] - An unique identifier for the document, used for offline mode.
 * @property {function} [onLoadingProgress] - A callback function for loading progress, function signature function(percent) {}.
 * @property {function} [onError] - A callback function that will be called when error occurs in the process of creating a document. function signature function(e) {}
 * @property {Promise.<any>} [workerTransportPromise] - The workerTransportPromise that should be used to load the document.
 * @property {string|function} [password] - A password string or a function of the form function(callback) where callback is of the form function(password). This 'password' function will be called when a password is required to load a PDF document and should call the callback with the retrieved password.
 * @property {string} [filename] - A filename that is used for the downloaded file, and for determining the extension when options.extension isn't used.
 * @property {string} [extension] - Used for specifying the extension of the document to be loaded. This is necessary if the URL/path does not contain the file extension or if you're loading a Blob/File.
 * @property {object} [customHeaders] - An object containing custom HTTP headers to use when retrieving the document from the specified url. For example: {'Authorization' : 'Basic dXNlcm5hbWU6cGFzc3dvcmQ='}.
 * @property {boolean} [useDownloader] - A boolean indicating whether Downloader should be used on urls (PDF only).
 * @property {boolean} [withCredentials] - Whether to set the withCredentials property on the XMLHttpRequest.
 * @property {Array.<object>} [pageSizes] - An array of objects in the shape of { width: number, height: number }. Used to determine the page sizes when loading an image file.
 * @property {string} [backendType] - A string representing the "backend type" for rendering PDF documents. Pass "asm" to force the use of the ASM.js worker, "ems" to force the use of the WebAssembly worker (or ASM.js on non-wasm browsers) or "wasm-threads" to use threaded WebAssembly.
 * @property {object} [xodOptions] - An object that contains the options for a XOD document.
 * @property {boolean} [xodOptions.decrypt] - Function to be called to decrypt a part of the XOD file. For default XOD AES encryption pass Core.Encryption.decrypt.
 * @property {boolean} [xodOptions.decryptOptions] - An object with options for the decryption e.g. {p: "pass", type: "aes"} where is p is the password.
 * @property {boolean} [xodOptions.streaming] - A boolean indicating whether to use http or streaming PartRetriever, it is recommended to keep streaming false for better performance.
 * @property {boolean} [xodOptions.azureWorkaround] - Whether or not to workaround the issue of Azure not accepting range requests of a certain type. Enabling the workaround will add an extra HTTP request of overhead but will still allow documents to be loaded from other locations.
 * @property {boolean} [xodOptions.startOffline] - Whether to start loading the document in offline mode or not. This can be set to true if the document had previously been saved to an offline database using WebViewer APIs. You'll need to use this option to load from a completely offline state.
 * @property {string} [webviewerServerURL] - A URL to the WebViewer server drop-in backend https://docs.apryse.com/documentation/web/guides/wv-server-deployment.
 * @property {boolean} [fallbackToClientSide] - A boolean indicating whether to fall back to client side rendering when WebViewer server fails
 * @property {string} [cacheKey] - A key that will be used for caching the document on WebViewer Server.
 * @property {boolean} [forceClientSideInit] - If set to true then when loading a document using WebViewer Server the document will always switch to client only rendering allowing page manipulation and the full API to be used.
 * @property {boolean} [loadAsPDF] - If set to true then this will convert any office file passed in into a PDF document
 * @property {number} [customHandlerId] - A field used to specify PDFTron custom security handler. Its value needs to be an integer in [0, 0xFFFFFFFF].
 * @property {string} [type] - A custom document type. If set then the document type is not determined from the file extension.
 * @property {object} [customDocParams] - An object used to pass properties down to a custom document type.
 * @property {boolean} [isRelativePath] - If true, source will be considered as relative path/url.
 * @property {boolean} [showInvalidBookmarks] - Default is true. If true, will retrieve invalid bookmarks
 * @property {object} [officeOptions] - An object that contains the options for an Office document.
 * @property {boolean} [enableOfficeEditing] - If true, will load office files with editing capabilities.
 * @property {Core.TemplateData} [officeOptions.templateValues] - If set, will perform template replacement with the data specified by this parameter.
 * @property {Core.TemplateOptions} [officeOptions.templateOptions] - If set, it will interpret the office document as a template document and compile all of the template tags in the document using the provided options.
 * @property {boolean} [officeOptions.doTemplatePrep] - If set, it will interpret the office document as a template document and compile all of the template tags in the document.
 * @property {boolean} [officeOptions.disableBrowserFontSubstitution] - By default, office viewing takes a lightweight approach to font substitution, allowing the browser to select fonts when they are not embedded in the document itself. While this means that WebViewer has access to all the fonts on the user's system, it also means that an office document may have a different "look" on different systems (depending on the fonts available) and when it is converted to PDF (as the PDF conversion routine cannot obtain low-level access to user fonts, for security reasons). disableBrowserFontSubstitution prevents this browser substitution, forcing the WebViewer backend to handle all fonts. This means that viewing and conversion to PDF will be 100% consistent from system-to-system, at the expense of a slightly slower initial viewing time and higher bandwidth usage. Using https://docs.apryse.com/documentation/web/faq/self-serve-substitute-fonts/ along with this option allows you to fully customize the substitution behaviour for all office files.
 * @property {object} [officeOptions.formatOptions] - An object that contains formatting options for an Office document. Same options as allowed here Core.PDFNet.Convert.OfficeToPDFOptions.
 * @property {boolean} [officeOptions.formatOptions.hideTotalNumberOfPages] - If true will hide total number of pages from page number labels (i.e, Page 1, Page 2, vs Page 1 of 2, Page 2 of 2)
 * @property {boolean} [officeOptions.formatOptions.applyPageBreaksToSheet] - If true will split Excel worksheets into pages so that the output resembles print output.
 * @property {boolean} [officeOptions.formatOptions.displayChangeTracking] - If true will display office change tracking markup present in the document (i.e, red strikethrough of deleted content and underlining of new content). Otherwise displays the resolved document content, with no markup. Defaults to true.
 * @property {boolean} [officeOptions.formatOptions.displayHiddenText] - If true will display hidden text in document. Otherwise hidden text will be not be shown. Defaults to false.
 * @property {number} [officeOptions.formatOptions.excelDefaultCellBorderWidth] - Cell border width for table cells that would normally be drawn with no border. In units of points. Can be used to achieve a similar effect to the "show gridlines" display option within Microsoft Excel.
 * @property {number} [officeOptions.formatOptions.excelMaxAllowedCellCount] - An exception will be thrown if the number of cells in an Excel document is above the value. Used for early termination of resource intensive documents. Setting this value to 250000 will allow the vast majority of Excel documents to convert without issue, while keeping RAM usage to a reasonable level. By default there is no limit to the number of allowed cells.
 * @property {string} [officeOptions.formatOptions.locale] - Sets the value for Locale in the options object ISO 639-1 code of the current system locale. For example: 'en-US', 'ar-SA', 'de-DE', etc.
 */
const CreateDocumentOptions = {}

/**
 * Load document options.
 * @typedef Core.LoadDocumentOptions
 * @property {boolean} [loadAnnotations=true] - Whether to load document with or without annotations.
 * @property {string} [l] - Same as licenseKey.
 * @property {string} [licenseKey] - The license key for viewing PDF or Office files (PDF/Office only). You only need to use one of 'l' or 'licenseKey'.
 * @property {string} [docId] - An unique identifier for the document, used for offline mode.
 * @property {function} [onLoadingProgress] - A callback function for loading progress, function signature function(percent) {}.
 * @property {function} [onError] - A callback function that will be called when error occurs in the process of creating a document. function signature function(e) {}
 * @property {Promise.<any>} [workerTransportPromise] - The workerTransportPromise that should be used to load the document.
 * @property {string|function} [password] - A password string or a function of the form function(callback) where callback is of the form function(password). This 'password' function will be called when a password is required to load a PDF document and should call the callback with the retrieved password.
 * @property {string} [filename] - A filename that is used for the downloaded file, and for determining the extension when options.extension isn't used.
 * @property {string} [extension] - Used for specifying the extension of the document to be loaded. This is necessary if the URL/path does not contain the file extension or if you're loading a Blob/File.
 * @property {object} [customHeaders] - An object containing custom HTTP headers to use when retrieving the document from the specified url. For example: {'Authorization' : 'Basic dXNlcm5hbWU6cGFzc3dvcmQ='}.
 * @property {boolean} [useDownloader] - A boolean indicating whether Downloader should be used on urls (PDF only).
 * @property {boolean} [withCredentials] - Whether to set the withCredentials property on the XMLHttpRequest.
 * @property {Array.<object>} [pageSizes] - An array of objects in the shape of { width: number, height: number }. Used to determine the page sizes when loading an image file.
 * @property {string} [backendType] - A string representing the "backend type" for rendering PDF documents. Pass "asm" to force the use of the ASM.js worker, "ems" to force the use of the WebAssembly worker (or ASM.js on non-wasm browsers) or "wasm-threads" to use threaded WebAssembly.
 * @property {object} [xodOptions] - An object that contains the options for a XOD document.
 * @property {boolean} [xodOptions.decrypt] - Function to be called to decrypt a part of the XOD file. For default XOD AES encryption pass Core.Encryption.decrypt.
 * @property {boolean} [xodOptions.decryptOptions] - An object with options for the decryption e.g. {p: "pass", type: "aes"} where is p is the password.
 * @property {boolean} [xodOptions.streaming] - A boolean indicating whether to use http or streaming PartRetriever, it is recommended to keep streaming false for better performance.
 * @property {boolean} [xodOptions.azureWorkaround] - Whether or not to workaround the issue of Azure not accepting range requests of a certain type. Enabling the workaround will add an extra HTTP request of overhead but will still allow documents to be loaded from other locations.
 * @property {boolean} [xodOptions.startOffline] - Whether to start loading the document in offline mode or not. This can be set to true if the document had previously been saved to an offline database using WebViewer APIs. You'll need to use this option to load from a completely offline state.
 * @property {string} [webviewerServerURL] - A URL to the WebViewer server drop-in backend https://docs.apryse.com/documentation/web/guides/wv-server-deployment.
 * @property {boolean} [fallbackToClientSide] - A boolean indicating whether to fall back to client side rendering when WebViewer server fails
 * @property {string} [cacheKey] - A key that will be used for caching the document on WebViewer Server.
 * @property {boolean} [forceClientSideInit] - If set to true then when loading a document using WebViewer Server the document will always switch to client only rendering allowing page manipulation and the full API to be used.
 * @property {boolean} [loadAsPDF] - If set to true then this will convert any office file passed in into a PDF document
 * @property {number} [customHandlerId] - A field used to specify PDFTron custom security handler. Its value needs to be an integer in [0, 0xFFFFFFFF].
 * @property {string} [type] - A custom document type. If set then the document type is not determined from the file extension.
 * @property {object} [customDocParams] - An object used to pass properties down to a custom document type.
 * @property {boolean} [isRelativePath] - If true, source will be considered as relative path/url.
 * @property {boolean} [showInvalidBookmarks] - Default is true. If true, will retrieve invalid bookmarks
 * @property {object} [officeOptions] - An object that contains the options for an Office document.
 * @property {boolean} [enableOfficeEditing] - If true, will load office files with editing capabilities.
 * @property {Core.TemplateData} [officeOptions.templateValues] - If set, will perform template replacement with the data specified by this parameter.
 * @property {Core.TemplateOptions} [officeOptions.templateOptions] - If set, it will interpret the office document as a template document and compile all of the template tags in the document using the provided options.
 * @property {boolean} [officeOptions.doTemplatePrep] - If set, it will interpret the office document as a template document and compile all of the template tags in the document.
 * @property {boolean} [officeOptions.disableBrowserFontSubstitution] - By default, office viewing takes a lightweight approach to font substitution, allowing the browser to select fonts when they are not embedded in the document itself. While this means that WebViewer has access to all the fonts on the user's system, it also means that an office document may have a different "look" on different systems (depending on the fonts available) and when it is converted to PDF (as the PDF conversion routine cannot obtain low-level access to user fonts, for security reasons). disableBrowserFontSubstitution prevents this browser substitution, forcing the WebViewer backend to handle all fonts. This means that viewing and conversion to PDF will be 100% consistent from system-to-system, at the expense of a slightly slower initial viewing time and higher bandwidth usage. Using https://docs.apryse.com/documentation/web/faq/self-serve-substitute-fonts/ along with this option allows you to fully customize the substitution behaviour for all office files.
 * @property {object} [officeOptions.formatOptions] - An object that contains formatting options for an Office document. Same options as allowed here Core.PDFNet.Convert.OfficeToPDFOptions.
 * @property {boolean} [officeOptions.formatOptions.hideTotalNumberOfPages] - If true will hide total number of pages from page number labels (i.e, Page 1, Page 2, vs Page 1 of 2, Page 2 of 2)
 * @property {boolean} [officeOptions.formatOptions.applyPageBreaksToSheet] - If true will split Excel worksheets into pages so that the output resembles print output.
 * @property {boolean} [officeOptions.formatOptions.displayChangeTracking] - If true will display office change tracking markup present in the document (i.e, red strikethrough of deleted content and underlining of new content). Otherwise displays the resolved document content, with no markup. Defaults to true.
 * @property {boolean} [officeOptions.formatOptions.displayHiddenText] - If true will display hidden text in document. Otherwise hidden text will be not be shown. Defaults to false.
 * @property {number} [officeOptions.formatOptions.excelDefaultCellBorderWidth] - Cell border width for table cells that would normally be drawn with no border. In units of points. Can be used to achieve a similar effect to the "show gridlines" display option within Microsoft Excel.
 * @property {number} [officeOptions.formatOptions.excelMaxAllowedCellCount] - An exception will be thrown if the number of cells in an Excel document is above the value. Used for early termination of resource intensive documents. Setting this value to 250000 will allow the vast majority of Excel documents to convert without issue, while keeping RAM usage to a reasonable level. By default there is no limit to the number of allowed cells.
 * @property {string} [officeOptions.formatOptions.locale] - Sets the value for Locale in the options object ISO 639-1 code of the current system locale. For example: 'en-US', 'ar-SA', 'de-DE', etc.
 */
const LoadDocumentOptions = {}
